#!/usr/bin/python3

import sys
import random

default = {
    "n": 10,
    "k": 5,
    "d": 5,
    "swap": 0,
    "maxw": 10**4,
}

def cmdlinearg(name):
    for arg in sys.argv:
        if arg.startswith(name + "="):
            return arg.split("=")[1]
    return default[name]

def main():

    random.seed(int(sys.argv[-1]))
    n = int(cmdlinearg("n"))
    k = int(cmdlinearg("k"))
    d = int(cmdlinearg("d"))
    swap = int(cmdlinearg("swap"))
    MAXW = int(cmdlinearg("maxw"))

    weights = [0] * n
    edges = []

    source = 0
    sink = n-1
    if swap:
        (source, sink) = (sink, source)
    
    weights[source] = random.randint(1, MAXW)
    weights[sink] = random.randint(1, MAXW)

    neighbours = random.sample(range(1, n-1), d)
    sn = set(neighbours)
    tree = [i for i in range(1, n-1) if i not in sn]

    # Get tree
    roots = [sink]
    distance = [0] * n
    leaf = [True] * n
    for i in tree:
        parent = roots[random.randrange(0, len(roots))]
        leaf[parent] = False
        edges.append([i, parent])

        weights[i] = random.randint(1, MAXW // 100)
        distance[i] = weights[i] + distance[parent]
        roots.append(i)
    leaves = []
    max_d = max(distance)
    for i in tree:
        if leaf[i]:
            leaves.append(i)
            weights[i] += max_d - distance[i]
    

    # Get neighbours
    w = random.randint(1, MAXW)
    for i in neighbours:
        edges.append([source, i])
        j = leaves[random.randrange(0, len(leaves))]
        edges.append([i, j])
        weights[i] = w

    print(n, len(edges), k)
    print(*weights)
    random.shuffle(edges)
    for (u, v) in edges:
        print(u+1, v+1)


    
if __name__ == "__main__":
    main()